"use strict";
/* eslint camelcase:0 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const githubauthquerystring_1 = __importDefault(require("githubauthquerystring"));
const ghapi = process.env.GITHUB_API || 'https://api.github.com';
// =================================
// Fetch Repository
/**
 * Fetch data for Repostiory from Repository Name
 * @param repoFullName Repostory name, such as `'bevry/getrepos'`
 */
async function getRepo(repoFullName) {
    try {
        const url = `${ghapi}/repos/${repoFullName}?${githubauthquerystring_1.default.fetch()}`;
        const resp = await cross_fetch_1.default(url, {
            headers: {
                Accept: 'application/vnd.github.v3+json'
            }
        });
        const data = (await resp.json());
        if (data && data.message)
            throw data.message;
        if (!data || !data.full_name)
            throw new Error('response was not a repository');
        return data;
    }
    catch (err) {
        return Promise.reject(githubauthquerystring_1.default.redact(err.message));
    }
}
exports.getRepo = getRepo;
/**
 * Fetch data for Repositories from Repository Names
 * @param repoFullNames Array of repository names, such as `['bevry/getcontributors', 'bevry/getrepos']`
 */
async function getRepos(repoFullNames) {
    return await Promise.all(repoFullNames.map(repoFullName => getRepo(repoFullName)));
}
exports.getRepos = getRepos;
// =================================
// Fetch from Search
/**
 * Fetch data for Repostiories from a Search, will iterate all subsequent pages
 * @param query The search query to send to GitHub, such as `@bevry/getcontributors @bevry/getrepos`
 */
async function getReposFromSearch(query, opts = {}) {
    if (opts.page == null)
        opts.page = 1;
    if (opts.pages == null)
        opts.pages = 10;
    if (opts.size == null)
        opts.size = 100;
    try {
        // fetch
        const url = `${ghapi}/search/repositories?page=${opts.page}&per_page=${opts.size}&q=${encodeURIComponent(query)}&${githubauthquerystring_1.default.fetch()}`;
        const resp = await cross_fetch_1.default(url, {
            headers: {
                Accept: 'application/vnd.github.v3+json'
            }
        });
        const data = (await resp.json());
        if (data && data.message)
            throw data.message;
        if (!data || !data.items || !Array.isArray(data.items))
            throw new Error('response was not the format we expected');
        if (data.items.length === 0)
            return [];
        const within = opts.pages === 0 || opts.page < opts.pages;
        if (data.items.length === opts.size && within)
            return data.items.concat(await getReposFromSearch(query, Object.assign({}, opts, { page: opts.page + 1 })));
        return data.items;
    }
    catch (err) {
        return Promise.reject(githubauthquerystring_1.default.redact(err.message));
    }
}
exports.getReposFromSearch = getReposFromSearch;
/**
 * Fetch data for Repostiories from Users
 * @param users Fetch repositories for these users, such as `['bevry', 'browserstate']`
 */
async function getReposFromUsers(users, opts = {}) {
    const query = users.map(name => `@${name}`).join('%20');
    return await getReposFromSearch(query, opts);
}
exports.getReposFromUsers = getReposFromUsers;
